/******************************************************************************
 * statman.hpp
 * Driver status visualization and additional playback control manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#ifndef STATMAN
#define STATMAN

/******************************************************************************
 * Layout
 ******************************************************************************/
#define WSTAT_RENDER_ENTRIES			2048
#define WSTAT_RENDER_LEN				(sizeof(mszRender) / sizeof(mszRender[0]))

/* |_File__Playback__About_____________________________________|
 * |                                                           |
 * |  _CoreTone_Info____________   _Wave_Output______________  |
 * | |                          | |    __        ___         | |
 * | | Render Rate : 48000Hz    | |___/  \      /   \     ___| |
 * | | Decode Rate : 240Hz      | |       \____/     \___/   | |
 * | | Update Tick : X          | |__________________________| |
 * | |__________________________|                              |
 * |                               _Sound_Effects____________  |
 * |  _Music_Volume_____________  |  ____  ____              | |
 * | |                          | | |    ||    |  PRI   PAN  | |
 * | |  0 ------------[]-- 127  | | | |> || [] | --[]- -[]-- | |
 * | | Soft              Loud   | | |____||____| L   H L   R | |
 * | |__________________________| |__________________________| |
 * |                                                           |
 * |                           ....                            |
 */
#define WSTAT_BASE_X					0
#define WSTAT_BASE_Y					0
#define WSTAT_WIDTH						512
#define WSTAT_HEIGHT					188

/* ---- CORETONE INFO ----
 */
#define WSTAT_CTINFO_ID					2010
#define WSTAT_CTINFO_X					(WSTAT_BASE_X + 16)
#define WSTAT_CTINFO_Y					(WSTAT_BASE_Y + 20)
#define WSTAT_CTINFO_WIDTH				288
#define WSTAT_CTINFO_HEIGHT				68

#define WSTAT_CTINFOL_ID				2011
#define WSTAT_CTINFOL_X					(WSTAT_CTINFO_X + 0)
#define WSTAT_CTINFOL_Y					(WSTAT_CTINFO_Y - 20)
#define WSTAT_CTINFOL_WIDTH				200
#define WSTAT_CTINFOL_HEIGHT			16

#define WSTAT_CTINFOB_ID				2012
#define WSTAT_CTINFOB_X					(WSTAT_CTINFO_X - 8)
#define WSTAT_CTINFOB_Y					(WSTAT_CTINFO_Y - 4)
#define WSTAT_CTINFOB_WIDTH				(WSTAT_CTINFO_WIDTH + 16)
#define WSTAT_CTINFOB_HEIGHT			(WSTAT_CTINFO_HEIGHT + 8)


/* ---- WAVE VUE ----
 */
#define WSTAT_WAVEVU_ID					2020
#define WSTAT_WAVEVU_X					(WSTAT_BASE_X + 328)
#define WSTAT_WAVEVU_Y					(WSTAT_BASE_Y + 20)
#define WSTAT_WAVEVU_WIDTH				160
#define WSTAT_WAVEVU_HEIGHT				68

#define WSTAT_WAVEVUL_ID				2021
#define WSTAT_WAVEVUL_X					(WSTAT_WAVEVU_X + 0)
#define WSTAT_WAVEVUL_Y					(WSTAT_WAVEVU_Y - 20)
#define WSTAT_WAVEVUL_WIDTH				200
#define WSTAT_WAVEVUL_HEIGHT			16

#define WSTAT_WAVEVUB_ID				2022
#define WSTAT_WAVEVUB_X					(WSTAT_WAVEVU_X - 8)
#define WSTAT_WAVEVUB_Y					(WSTAT_WAVEVU_Y - 4)
#define WSTAT_WAVEVUB_WIDTH				(WSTAT_WAVEVU_WIDTH + 16)
#define WSTAT_WAVEVUB_HEIGHT			(WSTAT_WAVEVU_HEIGHT + 8)


/* ---- MUSIC VOLUME ----
 */
#define WSTAT_MUSVOL_ID					2030
#define WSTAT_MUSVOL_X					(WSTAT_BASE_X + 16)
#define WSTAT_MUSVOL_Y					(WSTAT_BASE_Y + 116)
#define WSTAT_MUSVOL_WIDTH				160
#define WSTAT_MUSVOL_HEIGHT				30
#define WSTAT_MUSVOL_MIN				0
#define WSTAT_MUSVOL_MAX				127
#define WSTAT_MUSVOL_PAGE_SIZE			16

#define WSTAT_MUSVOL_SOFT_X				(WSTAT_MUSVOL_X + 0)
#define WSTAT_MUSVOL_SOFT_Y				(WSTAT_MUSVOL_Y + 32)
#define WSTAT_MUSVOL_LOUD_X				((WSTAT_MUSVOL_X + WSTAT_MUSVOL_WIDTH) - 24)
#define WSTAT_MUSVOL_LOUD_Y				(WSTAT_MUSVOL_Y + 32)

#define WSTAT_MUSVOLL_ID				2031
#define WSTAT_MUSVOLL_X					(WSTAT_MUSVOL_X + 0)
#define WSTAT_MUSVOLL_Y					(WSTAT_MUSVOL_Y - 20)
#define WSTAT_MUSVOLL_WIDTH				160
#define WSTAT_MUSVOLL_HEIGHT			16

#define WSTAT_MUSVOLB_ID				2032
#define WSTAT_MUSVOLB_X					(WSTAT_MUSVOL_X - 8)
#define WSTAT_MUSVOLB_Y					(WSTAT_MUSVOL_Y - 4)
#define WSTAT_MUSVOLB_WIDTH				(WSTAT_MUSVOLL_WIDTH + 16)
#define WSTAT_MUSVOLB_HEIGHT			(WSTAT_MUSVOL_HEIGHT + 24)


/* ---- SOUND EFFECT MANAGER ----
 */
#define WSTAT_SFXSEL_ID					2040
#define WSTAT_SFXSEL_X					(WSTAT_BASE_X + 200)
#define WSTAT_SFXSEL_Y					(WSTAT_BASE_Y + 116)
#define WSTAT_SFXSEL_WIDTH				288
#define WSTAT_SFXSEL_HEIGHT				46

#define WSTAT_SFXSELL_ID				2041
#define WSTAT_SFXSELL_X					(WSTAT_SFXSEL_X + 0)
#define WSTAT_SFXSELL_Y					(WSTAT_SFXSEL_Y - 20)
#define WSTAT_SFXSELL_WIDTH				200
#define WSTAT_SFXSELL_HEIGHT			16

#define WSTAT_SFXSELB_ID				2042
#define WSTAT_SFXSELB_X					(WSTAT_SFXSEL_X - 8)
#define WSTAT_SFXSELB_Y					(WSTAT_SFXSEL_Y - 4)
#define WSTAT_SFXSELB_WIDTH				(WSTAT_SFXSEL_WIDTH + 16)
#define WSTAT_SFXSELB_HEIGHT			(WSTAT_SFXSEL_HEIGHT + 8)

#define WSTAT_SFXSELP_ID				IDB_SFX_PLAY
#define WSTAT_SFXSELP_X					(WSTAT_SFXSEL_X + 0)
#define WSTAT_SFXSELP_Y					(WSTAT_SFXSEL_Y + 6)
#define WSTAT_SFXSELP_WIDTH				32
#define WSTAT_SFXSELP_HEIGHT			32

#define WSTAT_SFXSELS_ID				IDB_SFX_STOP
#define WSTAT_SFXSELS_X					(WSTAT_SFXSELP_X + 36)
#define WSTAT_SFXSELS_Y					(WSTAT_SFXSEL_Y + 6)
#define WSTAT_SFXSELS_WIDTH				32
#define WSTAT_SFXSELS_HEIGHT			32

#define WSTAT_SFXPRI_ID					2046
#define WSTAT_SFXPRI_X					(WSTAT_SFXSEL_X + 100)
#define WSTAT_SFXPRI_Y					(WSTAT_SFXSEL_Y + 0)
#define WSTAT_SFXPRI_WIDTH				80
#define WSTAT_SFXPRI_HEIGHT				30
#define WSTAT_SFXPRI_MIN				1
#define WSTAT_SFXPRI_MAX				127
#define WSTAT_SFXPRI_PAGE_SIZE			16

#define WSTAT_SFXPAN_ID					2047
#define WSTAT_SFXPAN_X					(WSTAT_SFXSEL_X + 208)
#define WSTAT_SFXPAN_Y					(WSTAT_SFXSEL_Y + 0)
#define WSTAT_SFXPAN_WIDTH				80
#define WSTAT_SFXPAN_HEIGHT				30
#define WSTAT_SFXPAN_MIN				0
#define WSTAT_SFXPAN_MAX				255
#define WSTAT_SFXPAN_PAGE_SIZE			32

#define WSTAT_SFXPRI_CAT_X				(WSTAT_SFXPRI_X - 20)
#define WSTAT_SFXPRI_CAT_Y				(WSTAT_SFXPRI_Y + 4)
#define WSTAT_SFXPRI_LABEL_X			(WSTAT_SFXPRI_X + 0)
#define WSTAT_SFXPRI_LABEL_Y			(WSTAT_SFXPRI_Y + 32)

#define WSTAT_SFXPAN_CAT_X				(WSTAT_SFXPAN_X - 24)
#define WSTAT_SFXPAN_CAT_Y				(WSTAT_SFXPAN_Y + 4)
#define WSTAT_SFXPAN_LABEL_X			(WSTAT_SFXPAN_X + 0)
#define WSTAT_SFXPAN_LABEL_Y			(WSTAT_SFXPAN_Y + 32)


/******************************************************************************
 * StatMan Class
 ******************************************************************************/
class StatMan {
	public:
		StatMan(HWND hParent, ToneMan *pToneMan);
		~StatMan(void);

		LRESULT CmdProc(HWND hWnd, UINT uMsg,
                        WPARAM wParam, LPARAM lParam);
		LRESULT TrackProc(HWND hWnd, UINT uMsg,
                          WPARAM wParam, LPARAM lParam);
		LRESULT PaintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin);
		LRESULT InvalProc(HWND hWnd);


		UINT muiCoreChannels;
		UINT muiRenderFreq,muiDecodeRate;
		UINT muiSamples,muiSampleLen;

	private:
		void RenderWave(HDC hDCm, COLORREF crLeft, COLORREF crRight,
					    COLORREF crMerge);


		HWND mhParent;
		ToneMan *mpToneMan;
		TCHAR mszRender[WSTAT_RENDER_ENTRIES];

		HWND mhBorder_ctInfo,mhLabel_ctInfo;
		INT midBorder_ctInfo,midLabel_ctInfo;

		HBITMAP mhWaveMap;
		HDC mhDCm;
		HWND mhBorder_waveVu,mhLabel_waveVu;
		INT midBorder_waveVu,midLabel_waveVu;

		HWND mhBorder_musVol,mhLabel_musVol,mhTrack_musVol;
		INT midBorder_musVol,midLabel_musVol,midTrack_musVol;
		DWORD dwLast_musVol;

		INT miActiveSfx;
		HICON mhIconPlay_A,mhIconPlay_G;
		HICON mhIconStop_A,mhIconStop_G;
		HWND mhPlay_sfxSel,mhStop_sfxSel,mhTrack_sfxPri,mhTrack_sfxPan;
		INT midPlay_sfxSel,midStop_sfxSel,midTrack_sfxPri,midTrack_sfxPan;
		INT miPlayPri,miPlayPan,miVol_Left,miVol_Right;

		HWND mhBorder_sfxSel,mhLabel_sfxSel;
		INT midBorder_sfxSel,midLabel_sfxSel;
};

#endif
